local AceGUI = LibStub("AceGUI-3.0");
local Spreadsheet = LibStub("LibSpreadsheet-1.0");


if ( GnomexcelDB == nil ) then
	GnomexcelDB = {
		books = {}
	};
end

Gnomexcel = {
	mainWin = nil;
	toolBox = nil;
	fileTree = nil;
	revision = tonumber(("$Rev: 04$"):match("(%d+)")) or "0 (dev copy)";
};

function Gnomexcel:CreateBook(title)
	local book = Spreadsheet:Book(title);
	book:StrictMode(true);
	local sheet = book:Add("Sheet 1");
	sheet.background = {1,1,1,0.4};

	for i = 1, 15 do
		sheet:AddColumn(
		{
			type="string",
			name = i,
			background={0.14,0.14,0.14},
			color={1,0.95,0.4}
		}
		);
	end
	for i = 1, 15 do
		sheet:AddRow("","","","","","","","","","","","","","","","","","","","","","","","","","");
	end
	sheet.isInteractive = true;
	book:EnableMaths(true);
	local hash = string.format("%8x-%8x-%8x", math.random(time()), math.random(time()/3), math.random(time()-400));
	book.hash = hash;

	table.insert(GnomexcelDB.books,
		{
		author = GetUnitName("Player"),
		realm = GetRealmName(),
		created = time(),
		file = book,
		hash = hash;
		}
	);
end

function Gnomexcel:DeleteBook()
	if ( Gnomexcel.book ~= nil ) then
		Gnomexcel.book:Hide();
		Gnomexcel.book:Close();
	end
	for key,book in pairs(GnomexcelDB.books) do
		if ( book.hash == Gnomexcel.book.hash ) then
			table.remove(GnomexcelDB.books, key);
			break;
		end
	end
	Gnomexcel:GetBooks();
end

function Gnomexcel:GetBooks()
	local tbl = {
		{
			value = "$$Function=NewBook$$",
			text = "Create a new book",
			icon = "Interface\\Icons\\INV_Misc_Book_06",
		},
		{
			value = "$$Empty$$",
			text = "   ",
			icon = "",
			disabled = true,
		},
		{
			value = "$$Empty$$",
			text = "|cffffffffBooks in your library:|r",
			icon = "",
			disabled = true,
		},

	};
	for key, book in pairs(GnomexcelDB.books) do
		table.insert(tbl, {
			value = "$$Book=" .. key .. "$$",
			text = book.file.title,
			icon = ("Interface\\Icons\\INV_Misc_Book_%02u"):format(math.random(0, 4) + 7);
		}
		);
	end
	Gnomexcel.fileTree:SetTree(tbl);
	Gnomexcel.fileTree:SetUserData("tree", tbl);

	Gnomexcel.fileTree:SetCallback("OnClick",
		function(caller, event, value)
            local subtree, subval = (value):match("%$(%a+)=([^%$]+)%$");
			if ( subtree == "Function" ) then
				if ( subval == "NewBook" ) then
					local frame = AceGUI:Create("Frame");
					frame:SetWidth(300);
					frame:SetHeight(100);
					frame:SetTitle("Create new book");
					local box = AceGUI:Create("EditBox");
					box:SetText("Enter the name of the book");
					frame:AddChild(box);
					frame:Show();
					box:SetCallback("OnEnterPressed",
						function(caller, event, title)
							Gnomexcel:CreateBook(title);
							Gnomexcel:GetBooks();
							frame:ReleaseChildren();
							frame:Release();
						end
					);
				end
			end
			if ( subtree == "Book" ) then
				if ( Gnomexcel.book ~= nil ) then
					Gnomexcel.book:Hide();
					Gnomexcel.book:Close();
				end
				Gnomexcel.book = Spreadsheet:Load(GnomexcelDB.books[tonumber(subval)].file);
				Gnomexcel.book:SetStyle("sheet");
				Gnomexcel.book:ShowSelector(true);
				Gnomexcel.book:Select();
				frame = Gnomexcel.book:Render();
				Gnomexcel.container:ReleaseChildren();
				frame:SetParent(Gnomexcel.container.frame);
				frame:SetPoint("TOPLEFT", Gnomexcel.container.frame, "TOPLEFT", 10, -10);
				frame:SetPoint("BOTTOMRIGHT", Gnomexcel.container.frame, "BOTTOMRIGHT", -10, 10);
				frame:Show();
			end
		end
	);
	Gnomexcel.fileTree:SetCallback("OnButtonEnter",
		function(caller, event, value)
			local subtree, subval = (value):match("%$(%a+)=([^%$]+)%$");
			if ( subtree == "Book" ) then
				if ( tonumber(subval) ~= 0 ) then
					local book = GnomexcelDB.books[tonumber(subval)];
					GameTooltip:SetOwner(caller.frame, "ANCHOR_TOPLEFT");
					GameTooltip:ClearLines();
					GameTooltip:SetText(book.file.title .. ":");
					GameTooltip:AddDoubleLine("|cff55DD11Author:|r", book.author .. " of " .. book.realm);
					GameTooltip:AddDoubleLine("|cff55DD11Created:|r", date(nil, book.created));
					GameTooltip:Show();
				end
			end
		end);
	Gnomexcel.fileTree:SetCallback("OnButtonLeave", function() GameTooltip:Hide(); end);
	Gnomexcel.fileTree:EnableButtonTooltips(false);
end

function Gnomexcel:MakeToolbox()

	local group = AceGUI:Create("SimpleGroup");
	group:SetWidth(440);
	group:SetLayout("Flow");

	local baleted = AceGUI:Create("Icon");
	baleted:SetWidth(80);
	baleted:SetImage([[Interface\Icons\INV_Misc_Bomb_07]]);
	baleted:SetImageSize(24,24);
	baleted:SetLabel("Delete book");
	baleted:SetCallback("OnClick", function() if ( Gnomexcel.book ) then Gnomexcel:DeleteBook() end end );
	group:AddChild(baleted);

	local addsheet = AceGUI:Create("Icon");
	addsheet:SetWidth(80);
	addsheet:SetLabel("Add a sheet");
	addsheet:SetImage([[Interface\Icons\INV_Misc_Bandage_10]]);
	addsheet:SetImageSize(24,24);
	addsheet:SetCallback("OnClick", function()
	if ( Gnomexcel.book ) then
		local sheet = Gnomexcel.book:Add(nil);
		sheet.background = {1,1,1,0.4};

	for i = 1, 15 do
		sheet:AddColumn(
		{
			type="string",
			name = i,
			background={0.14,0.14,0.14},
			color={1,0.95,0.4}
		}
		);
	end
	for i = 1, 15 do
		sheet:AddRow("","","","","","","","","","","","","","","","","","","","","","","","","","");
	end
	sheet.isInteractive = true;
		Gnomexcel.book:Render(); end end);
	group:AddChild(addsheet);

	local delsheet = AceGUI:Create("Icon");
	delsheet:SetWidth(80);
	delsheet:SetLabel("Delete sheet");
	delsheet:SetImage([[Interface\Icons\INV_Misc_Map_01]]);
	delsheet:SetImageSize(24,24);

	delsheet:SetCallback("OnClick", function() if ( Gnomexcel.book ) then Gnomexcel.book:Delete(nil); Gnomexcel.book:Render(); end end);
	group:AddChild(delsheet);


	local addcol = AceGUI:Create("Icon");
	addcol:SetImage([[Interface\Icons\Spell_ChargePositive]]);
	addcol:SetImageSize(24,24);
	addcol:SetWidth(90);
	addcol:SetLabel("Add a column");
	addcol:SetCallback("OnClick", function() if ( Gnomexcel.book ) then Gnomexcel.book.selectedSheet:AddColumn(
		{
			type="string",
			name = i,
			background={0.14,0.14,0.14},
			color={1,0.95,0.4}
		}
		); Gnomexcel.book:Render_ScrollFunction(); end end);
	group:AddChild(addcol);

	local addrow = AceGUI:Create("Icon");
	addrow:SetWidth(90);
	addrow:SetLabel("Add a row");
	addrow:SetImage([[Interface\Icons\Spell_ChargePositive]]);
	addrow:SetImageSize(24,24);
	addrow:SetCallback("OnClick", function() if ( Gnomexcel.book ) then Gnomexcel.book.selectedSheet:AddRow(""); Gnomexcel.book:Render_ScrollFunction(); end end);
	group:AddChild(addrow);
	Gnomexcel.toolBox:AddChild(group);

	group = AceGUI:Create("SimpleGroup");
	group:SetWidth(130);
	group:SetLayout("Flow");

	local dataType = AceGUI:Create("Dropdown");
	dataType:SetWidth(100);
	dataType:SetText("Data type:");
	dataType:SetList(
	{
		void = " ",
		number = "Numbers",
		currency = "Currency",
		string = "Text",
		boolean = "Booleans"
	}
	);
	dataType:SetCallback("OnValueChanged",
		function(caller, event, key)
			if ( Gnomexcel.book ) then
				if ( key ~= "void" ) then
					for k, cell in pairs(Gnomexcel.book.selectedSheet:GetSelectedCells()) do
						cell.t = key;
						cell.u = true;
					end
				end
				Gnomexcel.book:Render_Real();
			end
			dataType:SetValue("void");
			dataType:SetText("Data type:");
		end
		);
	dataType:SetHeight(24);
	group:AddChild(dataType);


	local decimals = AceGUI:Create("Dropdown");
	decimals:SetWidth(100);
	decimals:SetText("Decimals:");
	decimals:SetList( {" ", "0 ( 12345)", "1 (1234.5)", "2 (123.45)", "3 (12.345)", "4 (1.2345)" });
	decimals:SetCallback("OnValueChanged",
		function(caller, event, key)
			if ( Gnomexcel.book ) then
				if ( key ~= 0 ) then
					for k, cell in pairs(Gnomexcel.book.selectedSheet:GetSelectedCells()) do
						cell.d = tonumber(key) - 2;
						cell.u = true;
					end
				end
				Gnomexcel.book:Render_Real();
			end
			decimals:SetValue(" ");
			decimals:SetText("Decimals:");
		end
		);
	decimals:SetHeight(24);
	group:AddChild(decimals);

	local suffix = AceGUI:Create("Dropdown");
	suffix:SetWidth(80);
	suffix:SetText("Suffix:");
	local tbl = {};
	tbl.void = "";
	tbl[""] = "(none)";
	tbl['%'] = "%";
	tbl[' gold'] = "gold";
	suffix:SetList(tbl);
	suffix:SetCallback("OnValueChanged",
		function(caller, event, key)
			if ( Gnomexcel.book ) then
				if ( key ~= "void" ) then
					for k, cell in pairs(Gnomexcel.book.selectedSheet:GetSelectedCells()) do
						cell.s = key
						cell.u = true;
					end
				end
				Gnomexcel.book:Render_Real();
			end
			suffix:SetValue("void");
			suffix:SetText("Suffix:");
		end
		);
	suffix:SetHeight(24);
	group:AddChild(suffix);
	Gnomexcel.toolBox:AddChild(group);

	group = AceGUI:Create("SimpleGroup");
	group:SetWidth(90);
	group:SetLayout("Flow");

	local textColor = AceGUI:Create("ColorPicker");
	textColor:SetLabel("Text");
	textColor:SetWidth(89);
	textColor:SetColor(1,0.95,0.4);
	textColor:SetCallback("OnValueConfirmed",
		function(caller, event, r, g, b)
			if ( Gnomexcel.book ) then
				for k, cell in pairs(Gnomexcel.book.selectedSheet:GetSelectedCells()) do
					cell.c = {r,g,b,1};
					cell.u = true;
				end
				Gnomexcel.book:Render_Real();
			end
		end
	);
	group:AddChild(textColor);

	local background = AceGUI:Create("ColorPicker");
	background:SetLabel("Background");
	background:SetColor(0.14,0.14,0.14);
	background:SetWidth(89);
	background:SetCallback("OnValueConfirmed",
		function(caller, event, r, g, b)
			if ( Gnomexcel.book ) then
				for k, cell in pairs(Gnomexcel.book.selectedSheet:GetSelectedCells()) do
					cell.b = {r,g,b,1};
					cell.u = true;
				end
				Gnomexcel.book:Render_Real();
			end
		end
	);
	group:AddChild(background);

	Gnomexcel.toolBox:AddChild(group);
end


function Gnomexcel:MainWindow()
	local f = AceGUI:Create("Frame")
	f:SetCallback("OnClose",function(widget) f:Hide(); end)
	f:SetTitle("Gnomexcel rev. " .. Gnomexcel.revision);
	f:SetStatusText("Welcome!");
	f:SetLayout("Flow")
	f:SetWidth(700);
	f:SetAutoAdjustHeight(false);
	f:SetHeight(450);

	local toolBox = AceGUI:Create("SimpleGroup");
	toolBox.width= "fill";
	toolBox:SetHeight(100);
	toolBox:SetLayout("Flow")
	toolBox:SetAutoAdjustHeight(false);
	Gnomexcel.toolBox = toolBox;
	f:AddChild(toolBox);

	local fileTree = AceGUI:Create("TreeGroup");
	fileTree.width = "fill";
	fileTree.height = "fill";

	f:AddChild(fileTree);

	local container = AceGUI:Create("SimpleGroup");
	--container:SetTitle(book.title);
	--container.height = "fill";
	--container.width = "fill";

	fileTree:SetLayout("fill");
	fileTree:AddChild(container);

	f.frame:SetScript("OnShow", function() Gnomexcel:GetBooks(); end);
	Gnomexcel.mainWin = f;
	Gnomexcel.container = container;
	f:Hide();
	Gnomexcel.book = book;
	Gnomexcel.fileTree = fileTree;
	Gnomexcel:MakeToolbox();

	local t = AceGUI:Create("Label");
	t:SetText("Welcome to Gnomexcel! To get started, select one of your previous books or create a new one to start making spreadsheets!");
	t:SetWidth(400);
	container:AddChild(t);
end
do
	Gnomexcel:MainWindow();
	SLASH_GNOMEXCEL1, SLASH_GNOMEXCEL2 = '/gxl', '/gnomexcel';
function SlashCmdList.GNOMEXCEL(msg, editbox)
 Gnomexcel.mainWin:Show();
end

end
